function [b,trffkt,rever]=mdfa(K_mdfa,L_mdfa,lambda_mdfa,weight_func_mdfa,Lag_mdfa,Gamma_mdfa,expweight_mdfa,cutoff_mdfa,i1_mdfa,i2_mdfa,weight_constraint_mdfa)
%Ginters Buss 2011
% translated from Marc Wildi R I-mdfa7.txt code
% if useful, cite Ginters Buss, 2014.
% "Tracking economic activity in the euro area: Multivariate
% direct filter approach," OECD Journal: Journal of Business
% Cycle Measurement and Analysis, OECD Publishing,Centre for
% International Research on Economic Tendency Surveys, vol. 2014
% (2), pages 1-20. 
% I urge you to use the regularized version of this code, as in
% my paper to appear in Journal of Forecasting	 
    % weighting of amplitude function in stopband
    sizewf=size(weight_func_mdfa);
    omega_Gamma=fix(cutoff_mdfa*K_mdfa/pi);
    if (K_mdfa-omega_Gamma+1)>0
        weight_h=weight_func_mdfa.*([ones(1,omega_Gamma),(1:(K_mdfa-omega_Gamma+1)).^(expweight_mdfa/2)]'*ones(1,sizewf(2)));
    else
        weight_h=weight_func_mdfa.*(ones(1,K_mdfa+1)'*ones(1,sizewf(2)));
    end
    %plot(abs(weight_h))
    %Frequency zero receives half weight
    weight_h(1,:)=weight_h(1,:)/2;
    %DFT target variable
    weight_target=weight_h(:,1);
    %Rotate all DFT's such that weight_target is real (rotation does not alter mean-square error)
    weight_h=weight_h.*(exp(-1i*angle(weight_target))*ones(1,sizewf(2)));
    weight_target=weight_target.*exp(-1i*angle(weight_target));
    %DFT's explaining variables: target variable can be an explaining variable too, but it doesn't have to be one (counter-example: GDP)
    weight_h_exp=weight_h(:,2:end);
    sizewhe=size(weight_h_exp);
    % Regression weights for filter coefficient b_0 i.e. exp(1.i*0)
    X=real((exp(-1i*(0-Lag_mdfa)*pi*[0:K_mdfa]/K_mdfa)'*ones(1,sizewhe(2))).*weight_h_exp)+(sqrt(1+Gamma_mdfa*lambda_mdfa)*ones(1,sizewhe(2))).*(1i*imag((exp(-1i*(0-Lag_mdfa)*pi*[0:K_mdfa]/K_mdfa)'*ones(1,sizewhe(2))).*weight_h_exp));
    if i2_mdfa %don't set i2=T for Lag>0
        if ~i1_mdfa
            % Restriction i2_mdfa is true but i1_mdfa is false i.e. L_mdfa-1 freely determined parameters
            for l=2:L_mdfa-1
                % Regression weights explaining variables
                % Second order restriction is implemented: (l-1)/(L_mdfa-1)
                X=[X,real((exp(-1i*(l-1-Lag_mdfa)*pi*[0:K_mdfa]/K_mdfa)'*ones(1,sizewhe(2))).*weight_h_exp)-((l-1)/(L_mdfa-1))*real((exp(-1i*(L_mdfa-1-Lag_mdfa)*pi*[0:K_mdfa]/K_mdfa)'*ones(1,sizewhe(2))).*weight_h_exp)+ (sqrt(1+Gamma_mdfa*lambda_mdfa)*1i*ones(1,sizewhe(2))).*(imag((exp(-1i*(l-1-Lag_mdfa)*pi*[0:K_mdfa]/K_mdfa)'*ones(1,sizewhe(2))).*weight_h_exp)-((l-1)/(L_mdfa-1))*imag((exp(-1i*(L_mdfa-1-Lag_mdfa)*pi*[0:K_mdfa]/K_mdfa)'*ones(1,sizewhe(2))).*weight_h_exp))];
            end
            xtx=real(X)'*real(X)+imag(X)'*imag(X);
            % Solution
            size_x=size(X);
            bh=(inv(xtx)*(((real(X).*(real(weight_target)*ones(1,size_x(2))))'+(imag(X).*(imag(weight_target)*ones(1,size_x(2))))')*Gamma_mdfa))';
            %the last filter coefficient is a function of the previous ones through the second order restriction
            b=zeros(L_mdfa,length(weight_h_exp(1,:)));
            for k =1:length(weight_h_exp(1,:))
                bhh=bh([k+length(weight_h_exp(1,:))*[0:L_mdfa-2]]);
                b(:,k)=[bhh,-[1:L_mdfa-2]*(bhh(2:L_mdfa-1)')/(L_mdfa-1)]';
            end
        else
            X=X-(L_mdfa-1)*real((exp(-1i*(L_mdfa-2-Lag_mdfa)*pi*[0:K_mdfa]/K_mdfa)'*ones(1,sizewhe(2))).*weight_h_exp)+(L_mdfa-2)*real((exp(-1i*(L_mdfa-1-Lag_mdfa)*pi*[0:K_mdfa]/K_mdfa)'*ones(1,sizewhe(2))).*weight_h_exp)+(sqrt(1+Gamma_mdfa*lambda_mdfa)*ones(1,sizewhe(2))*1i).*((-1)*(L_mdfa-1)*imag((exp(-1i*(L_mdfa-2-Lag_mdfa)*pi*[0:K_mdfa]/K_mdfa)'*ones(1,sizewhe(2))).*weight_h_exp)+(L_mdfa-2)*imag((exp(-1i*(L_mdfa-1-Lag_mdfa)*pi*[0:K_mdfa]/K_mdfa)'*ones(1,sizewhe(2))).*weight_h_exp));
            she=sizewhe(2);
            % First and second order restriction: L_mdfa-2 freely determined parameters
            for l=2:L_mdfa-2
                % First and second order constraints are implemented
                realp=real((exp(-1i*(l-1-Lag_mdfa)*pi*[0:K_mdfa]/K_mdfa)'*ones(1,she)).*weight_h_exp)-(L_mdfa-l)*real((exp(-1i*(L_mdfa-2-Lag_mdfa)*pi*[0:K_mdfa]/K_mdfa)'*ones(1,she)).*weight_h_exp)+(L_mdfa-l-1)*real((exp(-1i*(L_mdfa-1-Lag_mdfa)*pi*[0:K_mdfa]/K_mdfa)'*ones(1,she)).*weight_h_exp);
                imagp=(sqrt(1+Gamma_mdfa*lambda_mdfa)*1i*ones(1,she)).*(imag((exp(-1i*(l-1-Lag_mdfa)*pi*[0:K_mdfa]/K_mdfa)'*ones(1,she)).*weight_h_exp)-(L_mdfa-l)*imag((exp(-1i*(L_mdfa-2-Lag_mdfa)*pi*[0:K_mdfa]/K_mdfa)'*ones(1,she)).*weight_h_exp)+(L_mdfa-l-1)*imag((exp(-1i*(L_mdfa-1-Lag_mdfa)*pi*[0:K_mdfa]/K_mdfa)'*ones(1,she)).*weight_h_exp));
                q=realp+imagp;
                X=[X,q]; 
            end
            xtx=real(X)'*real(X)+imag(X)'*imag(X);
            rpart=real(X)' * ((L_mdfa-1)*real((exp(-1i*(L_mdfa-2-Lag_mdfa)*pi*[0:K_mdfa]/K_mdfa)'*ones(1,she)).*(weight_h_exp.*(ones(sizewhe(1),1)*weight_constraint_mdfa)))-(L_mdfa-2)*real((exp(-1i*(L_mdfa-1-Lag_mdfa)*pi*[0:K_mdfa]/K_mdfa)'*ones(1,she)).*(weight_h_exp.*(ones(sizewhe(1),1)*weight_constraint_mdfa))));
            ipart=imag(X)' *((sqrt(1+Gamma_mdfa*lambda_mdfa)*ones(1,she)).*((L_mdfa-1)*imag((exp(-1i*(L_mdfa-2-Lag_mdfa)*pi*[0:K_mdfa]/K_mdfa)'*ones(1,she)).*(weight_h_exp.*(ones(sizewhe(1),1)*weight_constraint_mdfa)))-(L_mdfa-2)*imag((exp(-1i*(L_mdfa-1-Lag_mdfa)*pi*[0:K_mdfa]/K_mdfa)'*ones(1,she)).*(weight_h_exp.*(ones(sizewhe(1),1)*weight_constraint_mdfa)))));
            xtxy=sum(rpart+ipart,2);
            sx=size(X);
            bh=(inv(xtx)*((((real(X).*(real(weight_target)*ones(1,sx(2))))'+(imag(X).*(imag(weight_target)*ones(1,sx(2))))')*Gamma_mdfa)-xtxy))';
            % the last two filter weights are functions of the previous ones through the first and second order restrictions
            b=zeros(L_mdfa,length(weight_h_exp(1,:)));
            for k=1:length(weight_h_exp(1,:))
                bhh=bh(k+length(weight_h_exp(1,:))*[0:L_mdfa-3]);
                b(:,k)=[bhh,(L_mdfa-1)*weight_constraint_mdfa(k)-(L_mdfa-[1:L_mdfa-2])*bhh',-(L_mdfa-2)*weight_constraint_mdfa(k)+(L_mdfa-[2:L_mdfa-1])*bhh']';
            end
        end
    else
        % No filter constraint imposed
        if ~i1_mdfa
            for l=2:L_mdfa
                X=[X,real((exp(-1i*(l-1-Lag_mdfa)*pi*[0:K_mdfa]/K_mdfa)'*ones(1,sizewhe(2))).*weight_h_exp)+(sqrt(1+Gamma_mdfa*lambda_mdfa)*ones(1,sizewhe(2))*1i).*imag((exp(-1i*(l-1-Lag_mdfa)*pi*[0:K_mdfa]/K_mdfa)'*ones(1,sizewhe(2))).*weight_h_exp)];
            end
            xtx=real(X)'*real(X)+imag(X)'*imag(X);
            % MA filter weights
            sx=size(X);
            bh=(inv(xtx)*(((real(X).*(real(weight_target)*ones(1,sx(2))))'+(imag(X).*(imag(weight_target)*ones(1,sx(2))))')*Gamma_mdfa))';
            b=zeros(L_mdfa,length(weight_h_exp(1,:)));
            for k=1:length(weight_h_exp(1,:))
                bhh=bh(k+length(weight_h_exp(1,:))*[0:L_mdfa-1]);
                b(:,k)=bhh';
            end
        else
            % Only first order constraint imposed
            X=X-real((exp(-1i*(L_mdfa-1-Lag_mdfa)*pi*[0:K_mdfa]/K_mdfa)'*ones(1,sizewhe(2))).*weight_h_exp)-(sqrt(1+Gamma_mdfa*lambda_mdfa)*1i*ones(1,sizewhe(2))).*imag((exp(-1i*(L_mdfa-1-Lag_mdfa)*pi*[0:K_mdfa]/K_mdfa)'*ones(1,sizewhe(2))).*weight_h_exp);
            for l=2:L_mdfa-1
                X=[X,real((exp(-1i*(l-1-Lag_mdfa)*pi*[0:K_mdfa]/K_mdfa)'*ones(1,sizewhe(2))).*weight_h_exp)-real((exp(-1i*(L_mdfa-1-Lag_mdfa)*pi*[0:K_mdfa]/K_mdfa)'*ones(1,sizewhe(2))).*weight_h_exp)+(sqrt(1+Gamma_mdfa*lambda_mdfa)*1i*ones(1,sizewhe(2))).*imag((exp(-1i*(l-1-Lag_mdfa)*pi*[0:K_mdfa]/K_mdfa)'*ones(1,sizewhe(2))).*weight_h_exp)-(sqrt(1+Gamma_mdfa*lambda_mdfa)*1i*ones(1,sizewhe(2))).*imag((exp(-1i*(L_mdfa-1-Lag_mdfa)*pi*[0:K_mdfa]/K_mdfa)'*ones(1,sizewhe(2))).*weight_h_exp)];
            end
            xtx=real(X)'*real(X)+imag(X)'*imag(X);
            xtxy=sum(real(X)'*real((exp(-1i*(L_mdfa-1-Lag_mdfa)*pi*[0:K_mdfa]/K_mdfa)'*ones(1,sizewhe(2))).*(weight_h_exp.*(ones(sizewhe(1),1)*weight_constraint_mdfa)))+imag(X)'*((sqrt(1+Gamma_mdfa*lambda_mdfa)*ones(1,sizewhe(2))).*imag((exp(-1i*(L_mdfa-1-Lag_mdfa)*pi*[0:K_mdfa]/K_mdfa)'*ones(1,sizewhe(2))).*(weight_h_exp.*(ones(sizewhe(1),1)*weight_constraint_mdfa)))),2);
            % MA filter weights
            sx=size(X);
            bh=(inv(xtx)*((((real(X).*(real(weight_target)*ones(1,sx(2))))'+(imag(X).*(imag(weight_target)*ones(1,sx(2))))')*Gamma_mdfa)-xtxy))';
            % the last weight is a function of the previous ones through the first order restriction
            b=ones(L_mdfa,length(weight_h_exp(1,:)));
            for k=1:length(weight_h_exp(1,:))
                bhh=bh(k+length(weight_h_exp(1,:)).*[0:L_mdfa-2]);
                b(:,k)=[bhh,weight_constraint_mdfa(k)-sum(bhh)]';
            end
        end
    end
    % transfer function
    trffkt=zeros(K_mdfa+1,length(weight_h_exp(1,:)));
    trffkth=trffkt;
    trffkt(1,:)=sum(b,1);
    trffkth(1,:)=trffkt(1,:);
    for j = 1:length(weight_h_exp(1,:))
        for k=0:K_mdfa
            trffkt(k+1,j)=exp(1i*k*[0:L_mdfa-1]*pi/K_mdfa)*b(:,j);
        end
    end
    trt=sum((trffkt.*(exp(-1i*(0-Lag_mdfa)*pi*[0:K_mdfa]/K_mdfa)'*ones(1,sizewhe(2)))).*weight_h_exp,2);
    rever=sum(abs(Gamma_mdfa.*weight_target-real(trt)-1i*sqrt(1+lambda_mdfa*Gamma_mdfa).*imag(trt)).^2)/(2*(K_mdfa+1).^2);
    %rever=sum(abs(Gamma_mdfa.*weight_target-trt).^2)/(2*(K_mdfa+1)^2);
    % plot(abs(trffkt))
    
